@@ -1,7 +1,7 @@
 // -*- c++ -*- (enables emacs c++ mode)
 //===========================================================================
 //
-// Copyright (C) 2000-2008 Yves Renard
+// Copyright (C) 2000-2010 Yves Renard
 //
 // This file is a part of GETFEM++
 //
@@ -348,9 +348,10 @@ namespace getfem {
   template <typename MAT, typename VECT>
   void asm_real_or_complex_1_param
   (MAT &M, const mesh_im &mim, const mesh_fem &mf_u, const mesh_fem &mf_data,
-   const VECT &A, const mesh_region &rg, const char *assembly_description) {
+   const VECT &A, const mesh_region &rg, const char *assembly_description,
+   const mesh_fem *mf_mult = 0) {
     asm_real_or_complex_1_param_
-      (M, mim, mf_u, mf_data, A, rg, assembly_description,
+      (M, mim, mf_u, mf_data, A, rg, assembly_description, mf_mult,
        typename gmm::linalg_traits<VECT>::value_type());
   }
 
@@ -359,11 +360,12 @@ namespace getfem {
   void asm_real_or_complex_1_param_
   (const MAT &M, const mesh_im &mim,  const mesh_fem &mf_u,
    const mesh_fem &mf_data, const VECT &A,  const mesh_region &rg,
-   const char *assembly_description, T) {
+   const char *assembly_description, const mesh_fem *mf_mult, T) {
     generic_assembly assem(assembly_description);
     assem.push_mi(mim);
     assem.push_mf(mf_u);
     assem.push_mf(mf_data);
+    if (mf_mult) assem.push_mf(*mf_mult);
     assem.push_data(A);
     assem.push_mat_or_vec(const_cast<MAT&>(M));
     assem.assembly(rg);
@@ -374,13 +376,13 @@ namespace getfem {
   void asm_real_or_complex_1_param_
   (MAT &M, const mesh_im &mim, const mesh_fem &mf_u, const mesh_fem &mf_data,
    const VECT &A, const mesh_region &rg,const char *assembly_description,
-   std::complex<T>) {
+   const mesh_fem *mf_mult, std::complex<T>) {
     asm_real_or_complex_1_param_(gmm::real_part(M),mim,mf_u,mf_data,
 				 gmm::real_part(A),rg,
-				 assembly_description, T());
+				 assembly_description, mf_mult, T());
     asm_real_or_complex_1_param_(gmm::imag_part(M),mim,mf_u,mf_data,
 				 gmm::imag_part(A),rg,
-				 assembly_description, T());
+				 assembly_description, mf_mult, T());
   }
 
   /** 
@@ -473,15 +475,19 @@ namespace getfem {
   void asm_source_term(const VECT1 &B, const mesh_im &mim, const mesh_fem &mf,
 		       const mesh_fem &mf_data, const VECT2 &F,
 		       const mesh_region &rg = mesh_region::all_convexes()) {
-    GMM_ASSERT1(mf_data.get_qdim() == 1,
-		"invalid data mesh fem (Qdim=1 required)");
+    GMM_ASSERT1(mf_data.get_qdim() == 1 ||
+		mf_data.get_qdim() == mf.get_qdim(),
+		"invalid data mesh fem (same Qdim or Qdim=1 required)");
 
     const char *st;
     if (mf.get_qdim() == 1)
       st = "F=data(#2); V(#1)+=comp(Base(#1).Base(#2))(:,j).F(j);";
-    else
+    else if (mf_data.get_qdim() == 1)
       st = "F=data(qdim(#1),#2);"
 	"V(#1)+=comp(vBase(#1).Base(#2))(:,i,j).F(i,j);";
+    else
+      st = "F=data(#2);"
+	"V(#1)+=comp(vBase(#1).vBase(#2))(:,i,j,i).F(j);";
     
     asm_real_or_complex_1_param(const_cast<VECT1 &>(B),mim,mf,mf_data,F,rg,st);
   }
@@ -497,10 +503,9 @@ namespace getfem {
 		       const mesh_region &rg = mesh_region::all_convexes()) {
     const char *st;
     if (mf.get_qdim() == 1)
-      st = "F=data(1); V(#1)+=comp(Base(#1)).F(1);";
+      st = "F=data(1); V(#1)+=comp(Base(#1))(:).F(i);";
     else
-      st = "F=data(qdim(#1));"
-	"V(#1)+=comp(vBase(#1))(:,i).F(i);";
+      st = "F=data(qdim(#1)); V(#1)+=comp(vBase(#1))(:,i).F(i);";
     
     asm_real_or_complex_1_param(const_cast<VECT1 &>(B),mim,mf,mf,F,rg,st);
   }
@@ -511,18 +516,24 @@ namespace getfem {
       @ingroup asm
    */
   template<typename VECT1, typename VECT2>
-  void asm_normal_source_term(VECT1 &B, const mesh_im &mim, const mesh_fem &mf,
+  void asm_normal_source_term(VECT1 &B, const mesh_im &mim,
+			      const mesh_fem &mf,
 			      const mesh_fem &mf_data, const VECT2 &F,
 			      const mesh_region &rg) {
-    GMM_ASSERT1(mf_data.get_qdim() == 1, "invalid data mesh_fem");
+    GMM_ASSERT1(mf_data.get_qdim() == 1 ||
+		mf_data.get_qdim() == mf.get_qdim(),
+		"invalid data mesh_fem (same Qdim or Qdim=1 required)");
 
     const char *st;
     if (mf.get_qdim() == 1)
       st = "F=data(mdim(#1),#2);"
 	"V(#1)+=comp(Base(#1).Base(#2).Normal())(:,j,k).F(k,j);";
-    else
+    else if (mf_data.get_qdim() == 1)
       st = "F=data(qdim(#1),mdim(#1),#2);"
 	"V(#1)+=comp(vBase(#1).Base(#2).Normal())(:,i,j,k).F(i,k,j);";
+    else
+      st = "F=data(mdim(#1),#2);"
+	"V(#1)+=comp(vBase(#1).vBase(#2).Normal())(:,i,j,i,k).F(k,j);";
 
     asm_real_or_complex_1_param(B, mim, mf, mf_data, F, rg, st);
   }
@@ -1061,9 +1072,9 @@ namespace getfem {
 				      const mesh_region &rg) {
     generic_assembly assem("Kr=data$1(1); Ki=data$2(1);"
 			   "m = comp(Base(#1).Base(#1)); "
-			   "M$1(#1,#1)+=sym(m(:,:).Kr(1) - "
+			   "M$1(#1,#1)+=sym(m(:,:).Kr(j) - "
 			   "comp(Grad(#1).Grad(#1))(:,i,:,i));"
-			   "M$2(#1,#1)+=sym(m(:,:).Ki(1));");
+			   "M$2(#1,#1)+=sym(m(:,:).Ki(j));");
     assem.push_mi(mim);
     assem.push_mf(mf_u);
     assem.push_data(K_squaredr);
@@ -1080,7 +1091,7 @@ namespace getfem {
 				      const mesh_region &rg) {
     generic_assembly assem("K=data(1);"
 			   "m = comp(Base(#1).Base(#1)); "
-			   "M$1(#1,#1)+=sym(m(:,:).K(1) - "
+			   "M$1(#1,#1)+=sym(m(:,:).K(j) - "
 			   "comp(Grad(#1).Grad(#1))(:,i,:,i));");
     assem.push_mi(mim);
     assem.push_mf(mf_u);
@@ -1130,12 +1141,14 @@ namespace getfem {
 		"invalid data mesh fem (Qdim=1 required)");
     if (version & ASMDIR_BUILDH) {
       asm_mass_matrix(H, mim, mf_mult, mf_u, region);
-      gmm::clean(H, gmm::default_tol(magn_type())
-		 * gmm::mat_maxnorm(H) * magn_type(1000));
+//       gmm::clean(H, gmm::default_tol(magn_type()) //  remettre !
+// 		 * gmm::mat_maxnorm(H) * magn_type(1000));
     }
     if (version & ASMDIR_BUILDR)
       asm_source_term(R, mim, mf_mult, mf_r, r_data, region);
 
+    return; //  enlever !
+
     // Verifications and simplifications
 
     pfem pf_u, pf_r, pf_m;
