@@ -1,7 +1,7 @@
 // -*- c++ -*- (enables emacs c++ mode)
 //===========================================================================
 //
-// Copyright (C) 2004-2008 Yves Renard
+// Copyright (C) 2004-2010 Yves Renard
 //
 // This file is a part of GETFEM++
 //
@@ -204,7 +204,7 @@ namespace getfem {
 	cout << "starting computing tangent matrix" << endl;
       pb.compute_tangent_matrix();
       gmm::clear(dr);
-      gmm::copy(gmm::scaled(pb.residual(), T(-1)), b);
+      gmm::copy(gmm::scaled(pb.residual(), pb.scale_residual()), b);
       if (iter.get_noisy() > 1) cout << "starting linear solver" << endl;
       linear_solver(pb.tangent_matrix(), dr, b, iter_linsolv);
       if (iter.get_noisy() > 1) cout << "linear solver done" << endl;      
@@ -333,19 +333,24 @@ namespace getfem {
   */
   void standard_solve(model &md, gmm::iteration &iter,
 		      rmodel_plsolver_type lsolver,
-		      gmm::abstract_newton_line_search &ls);
+		      gmm::abstract_newton_line_search &ls,
+		      bool with_pseudo_potential = false);
 
   void standard_solve(model &md, gmm::iteration &iter,
 		      cmodel_plsolver_type lsolver,
-		      gmm::abstract_newton_line_search &ls);
+		      gmm::abstract_newton_line_search &ls,
+		      bool with_pseudo_potential = false);
   
   void standard_solve(model &md, gmm::iteration &iter,
-		      rmodel_plsolver_type lsolver);
+		      rmodel_plsolver_type lsolver,
+		      bool with_pseudo_potential = false);
 
   void standard_solve(model &md, gmm::iteration &iter,
-		      cmodel_plsolver_type lsolver);
+		      cmodel_plsolver_type lsolver,
+		      bool with_pseudo_potential = false);
 
-  void standard_solve(model &md, gmm::iteration &iter);
+  void standard_solve(model &md, gmm::iteration &iter,
+		      bool with_pseudo_potential = false);
 
 
 }
@@ -596,6 +601,8 @@ namespace getfem {
 
     TYPEDEF_MODEL_STATE_TYPES;
     typedef T_MATRIX MATRIX;
+    typedef typename gmm::linalg_traits<VECTOR>::value_type T;
+
 
     MODEL_STATE &MS;
     mdbrick_abstract<MODEL_STATE> &pb;
@@ -610,6 +617,8 @@ namespace getfem {
       }
     }
 
+    inline T scale_residual(void) const { return T(-1); }
+
     const T_MATRIX &tangent_matrix(void)
     { return MS.reduced_tangent_matrix(); }
     
@@ -629,7 +638,7 @@ namespace getfem {
       MS.unreduced_solution(dr, d);
       R alpha(1), res;
       
-      ls.init_search(gmm::vect_norm2(residual()), iter.get_iteration());
+      ls.init_search(MS.reduced_residual_norm(), iter.get_iteration());
       do {
 	alpha = ls.next_try();
 	gmm::add(stateinit, gmm::scaled(d, alpha), MS.state());
@@ -705,8 +714,7 @@ namespace getfem {
   standard_solve(MODEL_STATE &MS, mdbrick_abstract<MODEL_STATE> &problem,
 		 gmm::iteration &iter,
 		 typename useful_types<MODEL_STATE>::plsolver_type lsolver) {
-    gmm::default_newton_line_search ls(size_t(-1), 5.0/3.0,
-				       1.0/1000.0, 3.0/5.0, 1.6);
+    gmm::default_newton_line_search ls;
     standard_solve(MS, problem, iter, lsolver, ls);
   }
 
@@ -714,8 +722,7 @@ namespace getfem {
   template <typename MODEL_STATE> void
   standard_solve(MODEL_STATE &MS, mdbrick_abstract<MODEL_STATE> &problem,
 		 gmm::iteration &iter) {
-    gmm::default_newton_line_search ls(size_t(-1), 5.0/3.0,
-				       1.0/1000.0, 3.0/5.0, 1.6);
+    gmm::default_newton_line_search ls;
     standard_solve(MS, problem, iter, default_linear_solver(problem), ls);
   }
 
